@@ -66,6 +66,15 @@ if os.getenv("XDG_CONFIG_HOME"):
     tpl_lookup.directories.insert(0, path.join(os.getenv("XDG_CONFIG_HOME", ''), "pdoc"))
 
 
+# A surrogate so that the check in Module._link_inheritance()
+# "__pdoc__-overriden key {!r} does not exist" can pick the object up
+# (and not warn).
+# If you know how to keep the warning, but skip the object creation
+# altogether, please make it happen!
+class _BLACKLISTED_DUMMY:
+    pass
+
+
 class Context(dict):
     """
     The context object that maps all documented identifiers
@@ -90,7 +99,9 @@ def reset():
     _global_context.clear()
 
     # Clear LRU caches
-    for func in (_get_type_hints,):
+    for func in (_get_type_hints,
+                 _is_blacklisted,
+                 _is_whitelisted):
         func.cache_clear()
     for cls in (Doc, Module, Class, Function, Variable, External):
         for _, method in inspect.getmembers(cls):
@@ -121,7 +132,7 @@ def _get_config(**kwargs):
     config.update(kwargs)
 
     if 'search_query' in config:
-        warn('Option `search_query` has been depricated, use `google_search_query` instead',
+        warn('Option `search_query` has been deprecated. Use `google_search_query` instead',
              DeprecationWarning, stacklevel=2)
         config['google_search_query'] = config['search_query']
         del config['search_query']
@@ -253,16 +264,24 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
         tree = _init_tree
     else:
         try:
-            tree = ast.parse(inspect.getsource(doc_obj.obj))
+            # Maybe raise exceptions with appropriate message
+            # before using cleaned doc_obj.source
+            _ = inspect.findsource(doc_obj.obj)
+            tree = ast.parse(doc_obj.source)  # type: ignore
         except (OSError, TypeError, SyntaxError) as exc:
-            warn("Couldn't read PEP-224 variable docstrings from {!r}: {}".format(doc_obj, exc))
+            # Don't emit a warning for builtins that don't have source available
+            is_builtin = getattr(doc_obj.obj, '__module__', None) == 'builtins'
+            if not is_builtin:
+                warn("Couldn't read PEP-224 variable docstrings from {!r}: {}".format(doc_obj, exc),
+                     stacklevel=3 + int(isinstance(doc_obj, Class)))
             return {}, {}
 
         if isinstance(doc_obj, Class):
             tree = tree.body[0]  # ast.parse creates a dummy ast.Module wrapper
 
             # For classes, maybe add instance variables defined in __init__
-            for node in tree.body:
+            # Get the *last* __init__ node in case it is preceded by @overloads.
+            for node in reversed(tree.body):
                 if isinstance(node, ast.FunctionDef) and node.name == '__init__':
                     instance_vars, _ = _pep224_docstrings(doc_obj, _init_tree=node)
                     break
@@ -311,6 +330,7 @@ def _pep224_docstrings(doc_obj: Union['Module', 'Class'], *,
     return vars, instance_vars
 
 
+@lru_cache()
 def _is_whitelisted(name: str, doc_obj: Union['Module', 'Class']):
     """
     Returns `True` if `name` (relative or absolute refname) is
@@ -326,6 +346,7 @@ def _is_whitelisted(name: str, doc_obj: Union['Module', 'Class']):
     return False
 
 
+@lru_cache()
 def _is_blacklisted(name: str, doc_obj: Union['Module', 'Class']):
     """
     Returns `True` if `name` (relative or absolute refname) is
@@ -617,24 +638,32 @@ class Module(Doc):
             public_objs = []
             for name in self.obj.__all__:
                 try:
-                    public_objs.append((name, getattr(self.obj, name)))
+                    obj = getattr(self.obj, name)
                 except AttributeError:
                     warn("Module {!r} doesn't contain identifier `{}` "
                          "exported in `__all__`".format(self.module, name))
+                if not _is_blacklisted(name, self):
+                    obj = inspect.unwrap(obj)
+                public_objs.append((name, obj))
         else:
             def is_from_this_module(obj):
                 mod = inspect.getmodule(inspect.unwrap(obj))
                 return mod is None or mod.__name__ == self.obj.__name__
 
-            public_objs = [(name, inspect.unwrap(obj))
+            public_objs = [(name, (_BLACKLISTED_DUMMY
+                                   if _is_blacklisted(name, self) else
+                                   inspect.unwrap(obj)))
                            for name, obj in inspect.getmembers(self.obj)
                            if ((_is_public(name) or _is_whitelisted(name, self)) and
-                               (is_from_this_module(obj) or name in var_docstrings))]
+                               (_is_blacklisted(name, self) or  # skips unwrapping that follows
+                                is_from_this_module(obj) or name in var_docstrings))]
             index = list(self.obj.__dict__).index
             public_objs.sort(key=lambda i: index(i[0]))
 
         for name, obj in public_objs:
-            if _is_function(obj):
+            if obj is _BLACKLISTED_DUMMY:
+                self.doc[name] = Variable(name, self, 'dummy', obj=obj)
+            elif _is_function(obj):
                 self.doc[name] = Function(name, self, obj)
             elif inspect.isclass(obj):
                 self.doc[name] = Class(name, self, obj)
@@ -903,8 +932,8 @@ def _getmembers_all(obj: type) -> List[Tuple[str, Any]]:
     # Add keys from bases
     for base in mro:
         names.update(base.__dict__.keys())
-    # Add members for which type annotations exist
-    names.update(_get_type_hints(obj).keys())
+        # Add members for which type annotations exist
+        names.update(getattr(obj, '__annotations__', {}).keys())
 
     results = []
     for name in names:
@@ -948,7 +977,9 @@ class Class(Doc):
         # Use only own, non-inherited annotations (the rest will be inherited)
         annotations = getattr(self.obj, '__annotations__', {})
 
-        public_objs = [(_name, inspect.unwrap(obj))
+        public_objs = [(_name, (_BLACKLISTED_DUMMY
+                                if _is_blacklisted(_name, self) else
+                                inspect.unwrap(obj)))
                        for _name, obj in _getmembers_all(self.obj)
                        # Filter only *own* members. The rest are inherited
                        # in Class._fill_inheritance()
@@ -974,7 +1005,9 @@ class Class(Doc):
 
         # Convert the public Python objects to documentation objects.
         for name, obj in public_objs:
-            if _is_function(obj):
+            if obj is _BLACKLISTED_DUMMY:
+                self.doc[name] = Variable(name, self.module, 'dummy', obj=obj, cls=self)
+            elif _is_function(obj):
                 self.doc[name] = Function(
                     name, self.module, obj, cls=self)
             else:
@@ -1166,6 +1199,24 @@ class Class(Doc):
         del self._super_members
 
 
+def _formatannotation(annot):
+    """
+    Format annotation, properly handling NewType types
+
+    >>> import typing
+    >>> _formatannotation(typing.NewType('MyType', str))
+    'MyType'
+    """
+    module = getattr(annot, '__module__', '')
+    is_newtype = (getattr(annot, '__qualname__', '').startswith('NewType.') and
+                  module == 'typing')
+    if is_newtype:
+        return annot.__name__
+    if module.startswith('nptyping'):  # GH-231
+        return repr(annot)
+    return inspect.formatannotation(annot)
+
+
 class Function(Doc):
     """
     Representation of documentation for a function or method.
@@ -1235,42 +1286,24 @@ class Function(Doc):
     def return_annotation(self, *, link=None) -> str:
         """Formatted function return type annotation or empty string if none."""
         annot = ''
-        try:
-            try:
-                annot = _get_type_hints(self.obj)['return']
-            except Exception:
-                pass
-            else:
-                raise
-            try:
-                # This works mainly for non-property variables, and the rest are passed through
-                annot = _get_type_hints(cast(Class, self.cls).obj)[self.name]
-            except Exception:
-                pass
-            else:
-                raise
-            try:
+        for method in (
+                lambda: _get_type_hints(self.obj)['return'],
+                # Mainly for non-property variables
+                lambda: _get_type_hints(cast(Class, self.cls).obj)[self.name],
                 # global variables
-                annot = _get_type_hints(not self.cls and self.module.obj)[self.name]
-            except Exception:
-                pass
-            else:
-                raise
-            try:
-                annot = inspect.signature(self.obj).return_annotation
-            except Exception:
-                pass
-            else:
-                raise
-            try:
+                lambda: _get_type_hints(not self.cls and self.module.obj)[self.name],
+                lambda: inspect.signature(self.obj).return_annotation,
+                # Use raw annotation strings in unmatched forward declarations
+                lambda: cast(Class, self.cls).obj.__annotations__[self.name],
                 # Extract annotation from the docstring for C builtin function
-                annot = Function._signature_from_string(self).return_annotation
+                lambda: Function._signature_from_string(self).return_annotation,
+        ):
+            try:
+                annot = method()
             except Exception:
-                pass
+                continue
             else:
-                raise
-        except RuntimeError:  # Success.
-            pass
+                break
         else:
             # Don't warn on variables. The annotation just isn't available.
             if not isinstance(self, Variable):
@@ -1278,7 +1311,15 @@ class Function(Doc):
 
         if annot is inspect.Parameter.empty or not annot:
             return ''
-        s = inspect.formatannotation(annot).replace(' ', '\N{NBSP}')  # Better line breaks
+
+        if isinstance(annot, str):
+            s = annot
+        else:
+            s = _formatannotation(annot)
+            s = re.sub(r'\b(typing\.)?ForwardRef\((?P<quot>[\"\'])(?P<str>.*?)(?P=quot)\)',
+                       r'\g<str>', s)
+        s = s.replace(' ', '\N{NBSP}')  # Better line breaks in html signatures
+
         if link:
             from pdoc.html_helpers import _linkify
             s = re.sub(r'[\w\.]+', partial(_linkify, link=link, module=self.module), s)
@@ -1380,7 +1421,7 @@ class Function(Doc):
 
             formatted = p.name
             if p.annotation is not EMPTY:
-                annotation = inspect.formatannotation(p.annotation).replace(' ', '\N{NBSP}')
+                annotation = _formatannotation(p.annotation).replace(' ', '\N{NBSP}')
                 # "Eval" forward-declarations (typing string literals)
                 if isinstance(p.annotation, str):
                     annotation = annotation.strip("'")
