@@ -7,11 +7,14 @@ import importlib
 import inspect
 import os
 import os.path as path
+import json
 import re
 import sys
 import warnings
+from contextlib import contextmanager
+from functools import lru_cache
 from http.server import BaseHTTPRequestHandler, HTTPServer
-from typing import Sequence
+from typing import Dict, List, Sequence
 from warnings import warn
 
 import pdoc
@@ -314,28 +317,34 @@ def _quit_if_exists(m: pdoc.Module, ext: str):
             sys.exit(1)
 
 
-def recursive_write_files(m: pdoc.Module, ext: str, **kwargs):
-    assert ext in ('.html', '.md')
-    f = module_path(m, ext=ext)
-
-    dirpath = path.dirname(f)
-    if not os.access(dirpath, os.R_OK):
-        os.makedirs(dirpath)
-
+@contextmanager
+def _open_write_file(filename):
     try:
-        with open(f, 'w+', encoding='utf-8') as w:
-            if ext == '.html':
-                w.write(m.html(**kwargs))
-            elif ext == '.md':
-                w.write(m.text(**kwargs))
-        print(f)  # print created file path to stdout
+        with open(filename, 'w', encoding='utf-8') as f:
+            yield f
+            print(filename)  # print created file path to stdout
     except Exception:
         try:
-            os.unlink(f)
+            os.unlink(filename)
         except Exception:
             pass
         raise
 
+
+def recursive_write_files(m: pdoc.Module, ext: str, **kwargs):
+    assert ext in ('.html', '.md')
+    filepath = module_path(m, ext=ext)
+
+    dirpath = path.dirname(filepath)
+    if not os.access(dirpath, os.R_OK):
+        os.makedirs(dirpath)
+
+    with _open_write_file(filepath) as f:
+        if ext == '.html':
+            f.write(m.html(**kwargs))
+        elif ext == '.md':
+            f.write(m.text(**kwargs))
+
     for submodule in m.submodules():
         recursive_write_files(submodule, ext=ext, **kwargs)
 
@@ -362,6 +371,60 @@ def _warn_deprecated(option, alternative='', use_config_mako=False):
     warn(msg, DeprecationWarning, stacklevel=2)
 
 
+def _generate_lunr_search(modules: List[pdoc.Module],
+                          index_docstrings: bool,
+                          template_config: dict):
+    """Generate index.js for search"""
+
+    def trim_docstring(docstring):
+        return re.sub(r'''
+            \s+|                   # whitespace sequences
+            \s+[-=~]{3,}\s+|       # title underlines
+            ^[ \t]*[`~]{3,}\w*$|   # code blocks
+            \s*[`#*]+\s*|          # common markdown chars
+            \s*([^\w\d_>])\1\s*|   # sequences of punct of the same kind
+            \s*</?\w*[^>]*>\s*     # simple HTML tags
+        ''', ' ', docstring, flags=re.VERBOSE | re.MULTILINE)
+
+    def recursive_add_to_index(dobj):
+        info = {
+            'ref': dobj.refname,
+            'url': to_url_id(dobj.module),
+        }
+        if index_docstrings:
+            info['doc'] = trim_docstring(dobj.docstring)
+        if isinstance(dobj, pdoc.Function):
+            info['func'] = 1
+        index.append(info)
+        for member_dobj in getattr(dobj, 'doc', {}).values():
+            recursive_add_to_index(member_dobj)
+
+    @lru_cache()
+    def to_url_id(module):
+        url = module.url()
+        if url not in url_cache:
+            url_cache[url] = len(url_cache)
+        return url_cache[url]
+
+    index = []  # type: List[Dict]
+    url_cache = {}  # type: Dict[str, int]
+    for top_module in modules:
+        recursive_add_to_index(top_module)
+    urls = sorted(url_cache.keys(), key=url_cache.__getitem__)
+
+    main_path = args.output_dir
+    with _open_write_file(path.join(main_path, 'index.js')) as f:
+        f.write("URLS=")
+        json.dump(urls, f, indent=0, separators=(',', ':'))
+        f.write(";\nINDEX=")
+        json.dump(index, f, indent=0, separators=(',', ':'))
+
+    # Generate search.html
+    with _open_write_file(path.join(main_path, 'doc-search.html')) as f:
+        rendered_template = pdoc._render_template('/search.mako', **template_config)
+        f.write(rendered_template)
+
+
 def main(_args=None):
     """ Command-line entry point """
     global args
@@ -473,15 +536,13 @@ def main(_args=None):
 
     if args.pdf:
         _print_pdf(modules, **template_config)
+        import textwrap
         print("""
 PDF-ready markdown written to standard output.
                               ^^^^^^^^^^^^^^^
 Convert this file to PDF using e.g. Pandoc:
 
-    pandoc --metadata=title:"MyProject Documentation"               \\
-           --from=markdown+abbreviations+tex_math_single_backslash  \\
-           --pdf-engine=xelatex --variable=mainfont:"DejaVu Sans"   \\
-           --toc --toc-depth=4 --output=pdf.pdf  pdf.md
+{PANDOC_CMD}
 
 or using Python-Markdown and Chrome/Chromium/WkHtmlToPDF:
 
@@ -501,7 +562,7 @@ or using Python-Markdown and Chrome/Chromium/WkHtmlToPDF:
 
     wkhtmltopdf --encoding utf8 -s A4 --print-media-type pdf.html pdf.pdf
 
-or similar, at your own discretion.""",
+or similar, at your own discretion.""".format(PANDOC_CMD=textwrap.indent(_PANDOC_COMMAND, '    ')),
               file=sys.stderr)
         sys.exit(0)
 
@@ -517,6 +578,19 @@ or similar, at your own discretion.""",
             # Two blank lines between two modules' texts
             sys.stdout.write(os.linesep * (1 + 2 * int(module != modules[-1])))
 
+    lunr_config = pdoc._get_config(**template_config).get('lunr_search')
+    if lunr_config is not None:
+        _generate_lunr_search(
+            modules, lunr_config.get("index_docstrings", True), template_config)
+
+
+_PANDOC_COMMAND = '''\
+pandoc --metadata=title:"MyProject Documentation"               \\
+       --from=markdown+abbreviations+tex_math_single_backslash  \\
+       --pdf-engine=xelatex --variable=mainfont:"DejaVu Sans"   \\
+       --toc --toc-depth=4 --output=pdf.pdf  pdf.md\
+'''
+
 
 if __name__ == "__main__":
     main(parser.parse_args())
