@@ -6,7 +6,9 @@ import inspect
 import os
 import shutil
 import signal
+import subprocess
 import sys
+import typing
 import threading
 import unittest
 import warnings
@@ -18,12 +20,11 @@ from itertools import chain
 from random import randint
 from tempfile import TemporaryDirectory
 from time import sleep
+from types import ModuleType
 from unittest.mock import patch
 from urllib.error import HTTPError
 from urllib.request import Request, urlopen
 
-import typing
-
 import pdoc
 from pdoc import cli
 from pdoc.html_helpers import (
@@ -32,10 +33,17 @@ from pdoc.html_helpers import (
 )
 
 TESTS_BASEDIR = os.path.abspath(os.path.dirname(__file__) or '.')
-EXAMPLE_MODULE = 'example_pkg'
-
 sys.path.insert(0, TESTS_BASEDIR)
 
+EXAMPLE_MODULE = 'example_pkg'
+EXAMPLE_PDOC_MODULE = pdoc.Module(EXAMPLE_MODULE, context=pdoc.Context())
+PDOC_PDOC_MODULE = pdoc.Module(pdoc, context=pdoc.Context())
+
+EMPTY_MODULE = ModuleType('empty')
+EMPTY_MODULE.__pdoc__ = {}  # type: ignore
+with warnings.catch_warnings(record=True):
+    DUMMY_PDOC_MODULE = pdoc.Module(EMPTY_MODULE, context=pdoc.Context())
+
 T = typing.TypeVar("T")
 
 
@@ -243,16 +251,24 @@ class CliTest(unittest.TestCase):
             self._basic_html_assertions()
             self._check_files(exclude_patterns=['class="source"', 'Hidden'])
 
-    def test_html_with_google(self):
+    def test_google_search_query(self):
         with run_html(EXAMPLE_MODULE, config='google_search_query="anything"'):
             self._basic_html_assertions()
             self._check_files(include_patterns=['class="gcse-search"'])
 
-    def test_html_with_lunr(self):
+    def test_lunr_search(self):
         with run_html(EXAMPLE_MODULE, config='lunr_search={"fuzziness": 1}'):
-            files = self.PUBLIC_FILES + ["example_pkg/search.html", "example_pkg/index.js"]
+            files = self.PUBLIC_FILES + ["doc-search.html", "index.js"]
             self._basic_html_assertions(expected_files=files)
             self._check_files(exclude_patterns=['class="gcse-search"'])
+            self._check_files(include_patterns=['URLS=[\n"example_pkg/index.html",\n"example_pkg/'],
+                              file_pattern='index.js')
+            self._check_files(include_patterns=["'../doc-search.html#'"],
+                              file_pattern='example_pkg/index.html')
+            self._check_files(include_patterns=["'../doc-search.html#'"],
+                              file_pattern='example_pkg/module.html')
+            self._check_files(include_patterns=["'../../doc-search.html#'"],
+                              file_pattern='example_pkg/subpkg/index.html')
 
     def test_force(self):
         with run_html(EXAMPLE_MODULE):
@@ -379,6 +395,17 @@ class CliTest(unittest.TestCase):
         self.assertIn(pdoc_Doc_params.replace(' ', ''),
                       out.replace('>', '').replace('\n', '').replace(' ', ''))
 
+    @unittest.skipUnless('PDOC_TEST_PANDOC' in os.environ, 'PDOC_TEST_PANDOC not set/requested')
+    def test_pdf_pandoc(self):
+        with temp_dir() as path, \
+                chdir(path), \
+                redirect_streams() as (stdout, _), \
+                open('pdf.md', 'w') as f:
+            run('pdoc', pdf=None)
+            f.write(stdout.getvalue())
+            subprocess.run(pdoc.cli._PANDOC_COMMAND, shell=True, check=True)
+            self.assertTrue(os.path.exists('pdf.pdf'))
+
     def test_config(self):
         with run_html(EXAMPLE_MODULE, config='link_prefix="/foobar/"'):
             self._basic_html_assertions()
@@ -411,12 +438,28 @@ class CliTest(unittest.TestCase):
             run('.', skip_errors=None)
         self.assertIn('ZeroDivision', cm.warning.args[0])
 
+    @unittest.skipIf(sys.version_info < (3, 7), '__future__.annotations unsupported in <Py3.7')
+    def test_resolve_typing_forwardrefs(self):
+        # GH-245
+        with chdir(os.path.join(TESTS_BASEDIR, EXAMPLE_MODULE, '_resolve_typing_forwardrefs')):
+            with redirect_streams() as (out, _err):
+                run('postponed')
+            out = out.getvalue()
+            self.assertIn('bar', out)
+            self.assertIn('baz', out)
+            self.assertIn('dt', out)
+            self.assertIn('datetime', out)
+
+            with redirect_streams() as (out, _err):
+                run('evaluated')
+            out = out.getvalue()
+            self.assertIn('Set[Bar]', out)
+
 
 class ApiTest(unittest.TestCase):
     """
     Programmatic/API unit tests.
     """
-
     def setUp(self):
         pdoc.reset()
 
@@ -436,10 +479,9 @@ class ApiTest(unittest.TestCase):
 
     def test_Module_find_class(self):
         class A:
-            __module__ = None
+            pass
 
-        assert A.__module__ is None
-        mod = pdoc.Module(pdoc)
+        mod = PDOC_PDOC_MODULE
         self.assertIsInstance(mod.find_class(pdoc.Doc), pdoc.Class)
         self.assertIsInstance(mod.find_class(A), pdoc.External)
 
@@ -468,8 +510,7 @@ class ApiTest(unittest.TestCase):
                          [EXAMPLE_MODULE + '._private.module'])
 
     def test_instance_var(self):
-        pdoc.reset()
-        mod = pdoc.Module(EXAMPLE_MODULE)
+        mod = EXAMPLE_PDOC_MODULE
         var = mod.doc['B'].doc['instance_var']
         self.assertTrue(var.instance_var)
 
@@ -493,8 +534,7 @@ class ApiTest(unittest.TestCase):
             vars_dont = 0
             but_clss_have_doc = int
 
-        with self.assertWarns(UserWarning):
-            doc = pdoc.Class('C', pdoc.Module('pdoc'), C)
+        doc = pdoc.Class('C', DUMMY_PDOC_MODULE, C)
         self.assertEqual(doc.doc['vars_dont'].docstring, '')
         self.assertIn('integer', doc.doc['but_clss_have_doc'].docstring)
 
@@ -526,7 +566,7 @@ class ApiTest(unittest.TestCase):
         self.assertEqual(cls.doc['inherited'].refname, mod + '.B.inherited')
 
     def test_qualname(self):
-        module = pdoc.Module(EXAMPLE_MODULE)
+        module = EXAMPLE_PDOC_MODULE
         var = module.doc['var']
         cls = module.doc['B']
         nested_cls = cls.doc['C']
@@ -572,6 +612,30 @@ class ApiTest(unittest.TestCase):
             self.assertEqual(cm, [])
             self.assertNotIn('downloaded_modules', mod.doc)
 
+    @ignore_warnings
+    def test_dont_touch__pdoc__blacklisted(self):
+        class Bomb:
+            def __getattribute__(self, item):
+                raise RuntimeError
+
+        class D:
+            x = Bomb()
+            """doc"""
+            __qualname__ = 'D'
+
+        module = EMPTY_MODULE
+        D.__module__ = module.__name__  # Need to match is_from_this_module check
+        with patch.object(module, 'x', Bomb(), create=True), \
+             patch.object(module, '__pdoc__', {'x': False}):
+            mod = pdoc.Module(module)
+            pdoc.link_inheritance()
+            self.assertNotIn('x', mod.doc)
+        with patch.object(module, 'D', D, create=True), \
+             patch.object(module, '__pdoc__', {'D.x': False}):
+            mod = pdoc.Module(module)
+            pdoc.link_inheritance()
+            self.assertNotIn('x', mod.doc['D'].doc)
+
     def test__pdoc__invalid_value(self):
         module = pdoc.import_module(EXAMPLE_MODULE)
         with patch.object(module, '__pdoc__', {'B': 1}), \
@@ -656,7 +720,6 @@ class ApiTest(unittest.TestCase):
         self.assertEqual(result.name, nonexistent)
 
         # Ref by class __init__
-        mod = pdoc.Module(pdoc)
         self.assertIs(mod.find_ident('pdoc.Doc.__init__').obj, pdoc.Doc)
 
     def test_inherits(self):
@@ -709,7 +772,7 @@ class ApiTest(unittest.TestCase):
         class E(C):
             pass
 
-        mod = pdoc.Module(pdoc)
+        mod = DUMMY_PDOC_MODULE
         self.assertEqual([x.refname for x in pdoc.Class('A', mod, A).subclasses()],
                          [mod.find_class(C).refname])
         self.assertEqual([x.refname for x in pdoc.Class('B', mod, B).subclasses()],
@@ -759,7 +822,7 @@ class ApiTest(unittest.TestCase):
         self.assertIsInstance(module.find_ident('pdoc.Module'), pdoc.External)
 
     def test_Function_params(self):
-        mod = pdoc.Module(pdoc)
+        mod = PDOC_PDOC_MODULE
         func = pdoc.Function('f', mod,
                              lambda a, _a, _b=None: None)
         self.assertEqual(func.params(), ['a', '_a'])
@@ -824,6 +887,16 @@ class ApiTest(unittest.TestCase):
         self.assertEqual(pdoc.Function('bug130', mod, bug130_str_annotation).params(annotate=True),
                          ['a:\N{NBSP}str'])
 
+        # typed, NewType
+        CustomType = typing.NewType('CustomType', bool)
+
+        def bug253_newtype_annotation(a: CustomType):
+            return
+
+        self.assertEqual(
+            pdoc.Function('bug253', mod, bug253_newtype_annotation).params(annotate=True),
+            ['a:\N{NBSP}CustomType'])
+
         # builtin callables with signatures in docstrings
         from itertools import repeat
         self.assertEqual(pdoc.Function('repeat', mod, repeat).params(), ['object', 'times'])
@@ -838,7 +911,7 @@ class ApiTest(unittest.TestCase):
 
     @unittest.skipIf(sys.version_info < (3, 8), "positional-only arguments unsupported in < py3.8")
     def test_test_Function_params_python38_specific(self):
-        mod = pdoc.Module(pdoc)
+        mod = DUMMY_PDOC_MODULE
         func = pdoc.Function('f', mod, eval("lambda a, /, b: None"))
         self.assertEqual(func.params(), ['a', '/', 'b'])
 
@@ -846,22 +919,18 @@ class ApiTest(unittest.TestCase):
         self.assertEqual(func.params(), ['a', '/'])
 
     def test_Function_return_annotation(self):
-        import typing
-
         def f() -> typing.List[typing.Union[str, pdoc.Doc]]: pass
-        func = pdoc.Function('f', pdoc.Module(pdoc), f)
+        func = pdoc.Function('f', DUMMY_PDOC_MODULE, f)
         self.assertEqual(func.return_annotation(), 'List[Union[str,\N{NBSP}pdoc.Doc]]')
 
     @ignore_warnings
     def test_Variable_type_annotation(self):
-        import typing
-
         class Foobar:
             @property
             def prop(self) -> typing.Optional[int]:
                 pass
 
-        mod = pdoc.Module(pdoc)
+        mod = DUMMY_PDOC_MODULE
         cls = pdoc.Class('Foobar', mod, Foobar)
         self.assertEqual(cls.doc['prop'].type_annotation(), 'Union[int,\N{NBSP}NoneType]')
 
@@ -872,6 +941,8 @@ class ApiTest(unittest.TestCase):
             filename = os.path.join(path, 'module36syntax.py')
             with open(filename, 'w') as f:
                 f.write('''
+from typing import overload
+
 var: str = 'x'
 """dummy"""
 
@@ -879,15 +950,20 @@ class Foo:
     var: int = 3
     """dummy"""
 
-    def __init__(self):
-        self.var2: float = 1
-        """dummy"""
+    @overload
+    def __init__(self, var2: float):
+        pass
+
+    def __init__(self, var2):
+        self.var2: float = float(var2)
+        """dummy2"""
                 ''')
             mod = pdoc.Module(pdoc.import_module(filename))
             self.assertEqual(mod.doc['var'].type_annotation(), 'str')
             self.assertEqual(mod.doc['Foo'].doc['var'].type_annotation(), 'int')
             self.assertIsInstance(mod.doc['Foo'].doc['var2'], pdoc.Variable)
             self.assertEqual(mod.doc['Foo'].doc['var2'].type_annotation(), '')  # Won't fix
+            self.assertEqual(mod.doc['Foo'].doc['var2'].docstring, 'dummy2')
 
             self.assertIn('var: str', mod.text())
             self.assertIn('var: int', mod.text())
@@ -903,7 +979,6 @@ class Foo:
 
         class C:
             """foo"""
-
             def __init__(self):
                 """bar"""
 
@@ -916,14 +991,13 @@ class Foo:
 
         class F(typing.Generic[T]):
             """baz"""
-
             def __init__(self):
                 """bar"""
 
         class G(F[int]):
             """foo"""
 
-        mod = pdoc.Module(pdoc)
+        mod = DUMMY_PDOC_MODULE
         self.assertEqual(pdoc.Class('A', mod, A).docstring, """foo""")
         self.assertEqual(pdoc.Class('B', mod, B).docstring, """foo""")
         self.assertEqual(pdoc.Class('C', mod, C).docstring, """foo\n\nbar""")
@@ -938,7 +1012,7 @@ class Foo:
             def __init__(self, x):
                 pass
 
-        mod = pdoc.Module(pdoc)
+        mod = DUMMY_PDOC_MODULE
         self.assertEqual(pdoc.Class('C', mod, C).params(), ['x'])
         with patch.dict(mod.obj.__pdoc__, {'C.__init__': False}):
             self.assertEqual(pdoc.Class('C', mod, C).params(), [])
@@ -978,7 +1052,7 @@ class Foo:
 
     @unittest.skipIf(sys.version_info < (3, 6), reason="only deterministic on CPython 3.6+")
     def test_sorting(self):
-        module = pdoc.Module(EXAMPLE_MODULE)
+        module = EXAMPLE_PDOC_MODULE
 
         sorted_variables = module.variables()
         unsorted_variables = module.variables(sort=False)
@@ -1010,7 +1084,7 @@ class Foo:
     @ignore_warnings
     @unittest.skipIf(sys.version_info < (3, 6), 'variable type annotation unsupported in <Py3.6')
     def test_class_members(self):
-        module = pdoc.Module(EXAMPLE_MODULE)
+        module = DUMMY_PDOC_MODULE
 
         # GH-200
         from enum import Enum
@@ -1075,10 +1149,9 @@ reference: `package.foo`
 <p>ref with underscore: <code><a href="#pdoc._x_x_">_x_x_</a></code></p>
 <pre><code>code block
 </code></pre>
-
 <p>reference: <code><a href="/package.foo.ext">package.foo</a></code></p>'''
 
-        module = pdoc.Module(pdoc)
+        module = PDOC_PDOC_MODULE
         module.doc['_x_x_'] = pdoc.Variable('_x_x_', module, '')
 
         def link(dobj):
@@ -1124,11 +1197,11 @@ pdoc
         def link(dobj):
             return '<a>{}</a>'.format(dobj.qualname)
 
-        html = to_html(text, module=pdoc.Module(pdoc), link=link)
+        html = to_html(text, module=PDOC_PDOC_MODULE, link=link)
         self.assertEqual(html, expected)
 
     def test_to_html_refname_warning(self):
-        mod = pdoc.Module(EXAMPLE_MODULE)
+        mod = EXAMPLE_PDOC_MODULE
 
         def f():
             """Reference to some `example_pkg.nonexisting` object"""
@@ -1183,7 +1256,7 @@ pdoc
     def test_format_git_link(self):
         url = format_git_link(
             template='https://github.com/pdoc3/pdoc/blob/{commit}/{path}#L{start_line}-L{end_line}',
-            dobj=pdoc.Module(EXAMPLE_MODULE).find_ident('module.foo'),
+            dobj=EXAMPLE_PDOC_MODULE.find_ident('module.foo'),
         )
         self.assertIsInstance(url, str)
         self.assertRegex(url, r"https://github.com/pdoc3/pdoc/blob/[0-9a-f]{40}"
@@ -1193,7 +1266,7 @@ pdoc
 class Docformats(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls._module = pdoc.Module(pdoc)
+        cls._module = PDOC_PDOC_MODULE
         cls._docmodule = pdoc.import_module(EXAMPLE_MODULE)
 
     @staticmethod
@@ -1257,6 +1330,10 @@ description of <code>x1</code>, <code>x2</code>.</p>
 <dt><code><a>pdoc.Doc</a></code></dt>
 <dd>A class description that spans several lines.</dd>
 </dl>
+<h2 id="examples">Examples</h2>
+<pre><code class="language-python-repl">&gt;&gt;&gt; doctest
+...
+</code></pre>
 <h2 id="notes">Notes</h2>
 <p>Foo bar.</p>
 <h3 id="h3-title">H3 Title</h3>
@@ -1345,9 +1422,8 @@ that are relevant to the interface.</p>
 </dl>
 <h2 id="examples">Examples</h2>
 <p>Examples in doctest format.</p>
-<pre><code class="python-repl">&gt;&gt;&gt; a = [1,2,3]
+<pre><code class="language-python-repl">&gt;&gt;&gt; a = [1,2,3]
 </code></pre>
-
 <h2 id="todos">Todos</h2>
 <ul>
 <li>For module TODOs</li>
@@ -1367,22 +1443,19 @@ line2
 fenced code works
 always
 </code></pre>
-
 <h2 id="examples">Examples</h2>
-<pre><code class="python-repl">&gt;&gt;&gt; nbytes(100)
+<pre><code class="language-python-repl">&gt;&gt;&gt; nbytes(100)
 '100.0 bytes'
 line2
 </code></pre>
-
 <p>some text</p>
 <p>some text</p>
-<pre><code class="python-repl">&gt;&gt;&gt; another doctest
+<pre><code class="language-python-repl">&gt;&gt;&gt; another doctest
 line1
 line2
 </code></pre>
-
 <h2 id="example">Example</h2>
-<pre><code class="python-repl">&gt;&gt;&gt; f()
+<pre><code class="language-python-repl">&gt;&gt;&gt; f()
 Traceback (most recent call last):
     ...
 Exception: something went wrong
@@ -1435,9 +1508,8 @@ lines.</p>
         self.assertEqual(html, expected)
 
     def test_reST_include(self):
-        expected = '''<pre><code class="python">    x = 2
+        expected = '''<pre><code class="language-python">    x = 2
 </code></pre>
-
 <p>1
 x = 2
 x = 3
@@ -1450,7 +1522,7 @@ x =</p>'''
         # Ensure includes are resolved within docstrings already,
         # e.g. for `pdoc.html_helpers.extract_toc()` to work
         self.assertIn('Command-line interface',
-                      pdoc.Module(pdoc).docstring)
+                      self._module.docstring)
 
     def test_urls(self):
         text = """Beautiful Soup
@@ -1482,7 +1554,6 @@ Work <a href="http://foo/">like this</a> and <a href="ftp://bar">like that</a>.<
 <p>data:text/plain;base64,SGVsbG8sIFdvcmxkIQ%3D%3D</p>
 <pre><code>http://url.com
 </code></pre>
-
 <p><a href="https://google.com">https://google.com</a>
 <a href="https://en.wikipedia.org/wiki/Orange_(software)">\
 https://en.wikipedia.org/wiki/Orange_(software)</a>
